#!/usr/bin/env python3

from frame import FisheyeFrame
from frame import PinholeFrame
from detection import Detector
from coordinate import Transforms
import sys

import cv2
import rospy
from sensor_msgs.msg import CompressedImage
from sensor_msgs.msg import Image
from geometry_msgs.msg import PoseStamped
import numpy as np
from scipy.spatial.transform import Rotation


class ArucoPublisherNode:
    def __init__(self):
        path_calibration_camera = rospy.get_param("~calib_file", '/home/vdubois/Documents/Perso/Code/catkin_ws/src/girouette/src/resources/parameters_webcam_victor.txt')
        rospy.loginfo(f"loading camera calibration from {path_calibration_camera}")
        self.f = PinholeFrame.PinholeFrame(id_=0, parameters=path_calibration_camera)
        #self.f = FisheyeFrame.FisheyeFrame(id_=0, parameters=path_calibration_camera, balance=0.5)
        self.d = Detector.Detector()
        self.r_origin, self.t_origin = [[0,0,0]], [[0,0,0]]
        self.origin_id = 42
        self.initialized = False
        self.marker_sizes = {0: 0.07, 1: 0.07, 2: 0.07, 3: 0.07, 4: 0.07, 5: 0.07, 6: 0.07, 7: 0.07, 8: 0.07, 9: 0.07,
                             10: 0.07, 17: 0.06, 42: 0.10}
        self.image_pub_undistort = rospy.Publisher("/aruco_to_pose/debug/undistort/compressed",
                                         CompressedImage, queue_size=1)

        # subscribed Topic
        self.img_subscriber = rospy.Subscriber("/usb_cam/image_raw",
                                               Image, self.callback, queue_size=1)

    def core(self, distort):
        self.f.grabbed_frame = distort
        cv2.setNumThreads(2)
        #img = self.f.undistorted_scaled_frame() # FishEye only
        img = self.f.undistorted_frame()
        corners, ids = self.d.detect(img)
        poses = {}

        if ids is not None:
            for aruco_id, corner in zip(ids, corners):
                if aruco_id[0] == 17:
                    poses[aruco_id[0]] = self.d.find_marker_pose(corner, self.f.new_K, np.array([]), self.marker_sizes[aruco_id[0]])
                    rotation, translation = poses[aruco_id]
                    r = Rotation.from_matrix(rotation)
                    print("rotation = ")
                    print(r)

    def stop(self):
        self.f.close()

    def callback(self, ros_data):
        np_arr = np.fromstring(ros_data.data, np.uint8)
        print(np_arr)
        image_np = cv2.imdecode(np_arr, cv2.IMREAD_COLOR)# This step does not work with my log, probably because it is YUYV
        print(image_np)
        self.core(image_np)


def main(args):
    """ Initializes and cleanup ros node """
    rospy.init_node('aruco_pose_detector', anonymous=True)
    ArucoPublisherNode()
    try:
        rospy.spin()
    except KeyboardInterrupt:
        print("Shutting down ROS Image feature detector module")
    cv2.destroyAllWindows()


if __name__ == '__main__':
    main(sys.argv)
